#include <lfulib.h>
#include <samlib.h>

// global arrays & variables
static double	gradbasis[BASIS+1][3];	// gradient of basis functions
static double	basis[BASIS+1];			// basis functions
int		CoeffFlag;						// flag for computing coefficients -- needed for changes in head frame

// ECDLeadField()
void	ECDLeadField(
	double		Org[3],					// ECD coordinate
	HeaderInfo	*Header,				// MEG dataset header information
	ChannelInfo	*Channel,				// Channel[M] -- MEG channel information
	HULL		*OrigHull,				// cortical hull model
	double		**L						// L[M][3] -- output ECD primary sensor lead field matrix
) {
	static HULL		Hull;				// local inflated hull copy so that we don't compute voxels on the hull boundary
	static INT_PNT	*detector;			// detector[S+1] -- integration points & forward solutions for each cardinal moment
	double			Bd[3][3];
	double			x[3];				// x' vector
	double			y[3];				// y' vector
	double			Vd[3];				// dipole location
	double			Vs[3];				// detector location
	static double	**coeffs;			// coeffs[BASIS+1][S+1] -- correction coefficients
	static double	**fieldori;			// fieldori[S+1][3]
	static double	**correction;		// correction[S+1][3]
	static double	**coil;				// coil[C][3]
	static double	**Bp;				// Bp[M][3] -- primary sensor lead field
	static double	**Br;				// Br[R][3] -- reference sensor lead field
	double			angle;				// integration point in-plane angle
	double			Radius;				// integration point radius
	double			XDisp;				// x-displacement
	double			YDisp;				// y-displacement
	int				c;					// coil index
	int				cc;					// alternate coil index
	int				g;
	int				i;
	int				j;
	int				m;					// sensor index
	int				n;					// integration/detection point index
	int				found;				// found reference channel
	int				v;					// vector index
	int				w;					// another vector index
	extern int		CoeffFlag;			// flag for computing coefficients -- needed for changes in head frame
	static int		C;					// number of coils
	static int		S = 0;				// number of integration points

	// one-time memory allocation
	if(S == 0) {

		// count integration points & coils
		for(m=S=C=0; m<Header->NumChannels; m++)
			if((Channel[m].ChannelType == TYPE_REF_MAG || Channel[m].ChannelType == TYPE_REF_GRAD || Channel[m].ChannelType == TYPE_MEG) && Channel[m].Flag == TRUE)
				for(c=0; c<Channel[m].Geom.MEGSensor.NumCoils; c++) {
					C += 1;
					S += 3;
				}

		// allocate memory
		if((detector = (INT_PNT *)malloc((size_t)(S + 1) * sizeof(INT_PNT))) == NULL)
			allocfailed("detector[]");
		if((coeffs = (double **)malloc((size_t)(BASIS + 1) * sizeof(double *))) == NULL)
			allocfailed("coeffs[]");
		for(i=0; i<=BASIS; i++)
			if((coeffs[i] = (double *)malloc((size_t)(S + 1) * sizeof(double))) == NULL)
				allocfailed("coeffs[][]");
		if((fieldori = (double **)malloc((size_t)(S + 1) * sizeof(double *))) == NULL)
			allocfailed("fieldori[]");
		for(i=0; i<=S; i++)
			if((fieldori[i] = (double *)malloc((size_t)3 * sizeof(double))) == NULL)
				allocfailed("fieldori[][]");
		if((correction = (double **)malloc((size_t)(S + 1) * sizeof(double *))) == NULL)
			allocfailed("correction[]");
		for(i=0; i<=S; i++)
			if((correction[i] = (double *)malloc((size_t)3 * sizeof(double))) == NULL)
				allocfailed("correction[][]");
		if((coil = (double **)malloc((size_t)C * sizeof(double *))) == NULL)
			allocfailed("coil[]");
		for(i=0; i<C; i++)
			if((coil[i] = (double *)malloc((size_t)3 * sizeof(double))) == NULL)
				allocfailed("coil[][]");
		if((Bp = (double **)malloc((size_t)Header->NumPri * sizeof(double *))) == NULL)
			allocfailed("Bp[]");
		for(i=0; i<Header->NumPri; i++)
			if((Bp[i] = (double *)malloc((size_t)3 * sizeof(double))) == NULL)
				allocfailed("Bp[][]");
		if(Header->NumRef > 0) {
			if((Br = (double **)malloc((size_t)Header->NumRef * sizeof(double *))) == NULL)
				allocfailed("Br[]");
			for(i=0; i<Header->NumRef; i++)
				if((Br[i] = (double *)malloc((size_t)3 * sizeof(double))) == NULL)
					allocfailed("Br[][]");
		}

		// make inflated hull for Nolte solution
		Hull.nv = OrigHull->nv;
		for(v=X_; v<=Z_; v++)
			Hull.Vo[v] = OrigHull->Vo[v];
		Hull.scale = OrigHull->scale;
		if((Hull.vertex = (FIELD *)malloc((size_t)(Hull.nv + 1) * sizeof(FIELD))) == NULL)
			allocfailed("Hull.vertex[]");
		for(i=1; i<=Hull.nv; i++)
			for(v=X_; v<=Z_; v++) {
				Hull.vertex[i].p[v] = OrigHull->vertex[i].p[v] + INFLATE * OrigHull->vertex[i].v[v];	// assumes that normals are all outward-pointing
				Hull.vertex[i].v[v] = OrigHull->vertex[i].v[v];
			}

		// on 1st call, assure that CoeffFlag is TRUE
		CoeffFlag = TRUE;
	}

	// compute integration points (one-time, unless sensor frame changes)
	if(CoeffFlag == TRUE) {
		for(m=0; m<Header->NumChannels; m++) {
			if((Channel[m].ChannelType == TYPE_REF_MAG || Channel[m].ChannelType == TYPE_REF_GRAD || Channel[m].ChannelType == TYPE_MEG) && Channel[m].Flag == TRUE) {

				// evaluate all coil integration points
				for(c=0; c<Channel[m].Geom.MEGSensor.NumCoils; c++) {

					// set up three integration points in plane of coil
					OrthoPlane(Channel[m].Geom.MEGSensor.Coil[c].origin.v, x, y);	// solve for x' & y' defining plane of coil
					Channel[m].Geom.MEGSensor.Coil[c].NumInt = 3;
					Radius = M_SQRT1_2 * Channel[m].Geom.MEGSensor.Coil[c].radius;
					for(i=0; i<3; i++) {
						angle = (double)i * 2. * M_PI / 3.;
						XDisp = Radius * cos(angle);
						YDisp = Radius * sin(angle);
						for(v=X_; v<=Z_; v++)
							Channel[m].Geom.MEGSensor.Coil[c].B[i].p[v] = Channel[m].Geom.MEGSensor.Coil[c].origin.p[v] + x[v] * XDisp + y[v] * YDisp;
					}
					Channel[m].Geom.MEGSensor.Coil[c].Evaluated = TRUE;
				}
			}
		}

		// copy detection points to array
		for(m=0, n=1; m<Header->NumChannels; m++) {
			if((Channel[m].ChannelType == TYPE_REF_MAG || Channel[m].ChannelType == TYPE_REF_GRAD || Channel[m].ChannelType == TYPE_MEG) && Channel[m].Flag == TRUE) {
				for(c=0; c<Channel[m].Geom.MEGSensor.NumCoils; c++) {
					for(i=0; i<3; i++) {
						for(v=X_; v<=Z_; v++) {
							detector[n].Vi.p[v] = Channel[m].Geom.MEGSensor.Coil[c].B[i].p[v];
							detector[n].Vi.v[v] = Channel[m].Geom.MEGSensor.origin.v[v];
						}
						n++;
					}
				}
			}
		}

		// compute coefficients
		GetCoeffs(detector, S, &Hull, coeffs);
		CoeffFlag = FALSE;			// don't repeat unless sensor frame changes
	}

	// translate dipole source location to volume origin & compute correction for this dipole position
	for(v=X_; v<=Z_; v++)
		Vd[v] = Org[v] - Hull.Vo[v];
	GetBasis(Vd, Hull.scale, basis, gradbasis);

	// for all detection points
	for(i=1; i<=S; i++) {

		// translate detector location to conducting volume origin
		for(v=X_; v<=Z_; v++)
			Vs[v] = detector[i].Vi.p[v] - Hull.Vo[v];

		// solve field for spherical volume conductor (origin of volume conductor)
		xbd(Vd, Vs, Bd);

		// for each cardinal moment (v)
		for(v=X_; v<=Z_; v++) {

			// field is dot product of field Bd with detector normal vector (w)
			for(w=X_, fieldori[i][v]=0.; w<=Z_; w++)
				fieldori[i][v] += Bd[v][w] * detector[i].Vi.v[w];

			// final result
			for(j=1, correction[i][v]=0.; j<=BASIS; j++)
				correction[i][v] -= gradbasis[j][v] * coeffs[j][i];
			detector[i].Bi[v] = fieldori[i][v] + correction[i][v];
		}
	}

	// integrate field within each coil
	for(i=1, c=0; i<=S; i+=3, c++)
		for(v=X_; v<=Z_; v++)
			for(j=0, coil[c][v]=0.; j<3; j++)
				coil[c][v] += detector[i+j].Bi[v] / 3.;

	// compute field for each sensor
	for(m=c=i=j=0; m<Header->NumChannels; m++)
		if((Channel[m].ChannelType == TYPE_REF_MAG || Channel[m].ChannelType == TYPE_REF_GRAD) && Channel[m].Flag == TRUE) {
			for(v=X_; v<=Z_; v++) {
				for(cc=0, Br[i][v]=0.; cc<Channel[m].Geom.MEGSensor.NumCoils; cc++)
					Br[i][v] += coil[c][v] * (double)Channel[m].Geom.MEGSensor.Coil[cc].SenseTurns;
				Br[i][v] /= (double)abs(Channel[m].Geom.MEGSensor.Coil[0].SenseTurns);
			}
			c += Channel[m].Geom.MEGSensor.NumCoils;
			i++;
		} else if(Channel[m].ChannelType == TYPE_MEG && Channel[m].Flag == TRUE) {
			for(v=X_; v<=Z_; v++) {
				for(cc=0, Bp[j][v]=0.; cc<Channel[m].Geom.MEGSensor.NumCoils; cc++)
					Bp[j][v] += coil[c][v] * (double)Channel[m].Geom.MEGSensor.Coil[cc].SenseTurns;
				Bp[j][v] /= (double)abs(Channel[m].Geom.MEGSensor.Coil[0].SenseTurns);
			}
			c += Channel[m].Geom.MEGSensor.NumCoils;
			j++;
		}

	// if req'd, apply balance coefficients & output primary sensor lead field
	if(Header->NumRef > 0) {	// only if there are reference sensors
		for(m=0; m<Header->NumPri; m++) {
			j = Header->PsIndex[m];
			for(g=0; g<Channel[j].NumBal; g++) {
				for(i=0, found=FALSE; i<Header->NumRef; i++)
					if(Channel[j].Balance[g].RefChan == Header->RsIndex[i]) {
						found = TRUE;
						break;
					}
				if(found == FALSE)
					cleanup("couldn't find reference channel index");
				for(v=X_; v<=Z_; v++)
					Bp[m][v] -= Channel[j].Balance[g].Weight * Br[i][v];
			}
		}
	}
	for(m=0; m<Header->NumPri; m++)
		for(v=X_; v<=Z_; v++)
			L[m][v] = Bp[m][v];
}
