#include <lfulib.h>
#include <samlib.h>


// global arrays
static double	fabrnm[ORDER+1][ORDER+1][3];
static double	fabinm[ORDER+1][ORDER+1][3];
static double	frnm[ORDER+1][ORDER+1];
static double	finm[ORDER+1][ORDER+1];
static double	harmnorm[ORDER+1][ORDER+1];
static double	fak[2*ORDER+1];
static double	scalevec[ORDER+1];

// GetBasis()
void	GetBasis(
	double	Org[3],
	double	scale,
	double	basis[BASIS+1],
	double	gradbasis[BASIS+1][3]
) {
	double		Vs[3];			// spherical coordinates
	int			i;
	int			j;
	int			k;
	int			count;
	static int	once = FALSE;

	// one-time computation of harmomic norm (depends only on hull scale)
	if(once == FALSE) {
		for(i=1, fak[0]=1.; i<=2*ORDER; i++)
			fak[i] = (double)i * fak[i-1];
		for(i=1, scalevec[0]=1.; i<=ORDER; i++)
			scalevec[i] = scalevec[i-1] * scale;
		for(i=0; i<=ORDER; i++)
			for(j=0; j<=i; j++) {
				if(j == 0)
					harmnorm[i][j] = scalevec[i] * (double)i * sqrt(2. * fak[i+j] / fak[i-j] / (double)(2 * i + 1));
				else
					harmnorm[i][j] = scalevec[i] * (double)i * sqrt(fak[i+j] / fak[i-j] / (double)(2 * i + 1));
			}
		once = TRUE;
	}

	// convert cartesian to spherical coordinates
	CtoS(Org, Vs);

	rnynm(Vs, frnm, finm, fabrnm, fabinm);

	for(i=1; i<=ORDER; i++)
		for(j=0; j<=i; j++) {
			frnm[i][j] /= harmnorm[i][j];
			finm[i][j] /= harmnorm[i][j];
			for(k=0; k<3; k++) {
				fabrnm[i][j][k] /= harmnorm[i][j];
				fabinm[i][j][k] /= harmnorm[i][j];
			}
		}

	for(i=1, count=0; i<=ORDER; i++) {
		for(j=0; j<=i; j++) {
			count++;
			basis[count] = frnm[i][j];
			for(k=0; k<3; k++)
				gradbasis[count][k] = fabrnm[i][j][k];
		}
		for(j=1; j<=i; j++) {
			count++;
			basis[count] = finm[i][j];
			for(k=0; k<3; k++)
				gradbasis[count][k] = fabinm[i][j][k];
		}
	}
}
