#include <lfulib.h>

// global arrays
static double	ctc[BASIS+1][BASIS+1];		// C'C
static double	gradbasis[BASIS+1][3];		// gradient of basis functions
static double	achan[BASIS+1];
static double	basis[BASIS+1];				// basis functions
static double	gradbasisn[BASIS+1];		// gradients
static int		indx[BASIS+1];				// index for LU decomposition

// GetCoeffs()
void	GetCoeffs(
	INT_PNT	*detector,						// detector[S] -- array of detection points
	int		nchan,							// number of detection points
	HULL	*Hull,							// conducting volume vertices & normals
	double	**coeffs						// compensation coefficients
) {
	double			DipLoc[3];				// vertex location in hull origin fram
	double			SensLoc[3];				// sensor location in hull origin frame
	double			DipOri[3];				// vertex normal orienation
	double			SensOri[3];				// sensor normal orientation
	double			Bd[3][3];				// solution for dipole in spherical conductor
	double			x;
	double			d;
	static double	**ctb;					// C'b
	int				i;
	int				j;
	int				k;
	int				v;
	int				w;
	static int		M = 0;

	// one time memory allocation
	if(M == 0) {
		M = nchan;
		if((ctb = (double **)malloc((size_t)(BASIS + 1) * sizeof(double *))) == NULL)
			allocfailed("ctb[]");
		for(i=0; i<=BASIS; i++)
			if((ctb[i] = (double *)malloc((size_t)(M + 1) * sizeof(double))) == NULL)
				allocfailed("ctb[][]");
	}

	// initialize C'C & C'b to zero
	for(j=1; j<=BASIS; j++)
		for(k=1; k<=BASIS; k++)
			ctc[j][k] = 0.;
	for(j=1; j<=BASIS; j++)
		for(k=1; k<=M; k++)
			ctb[j][k] = 0.;

	// accumulate C'C & C'b over all conducting volume vertices
	for(i=1; i<=Hull->nv; i++) {

		// translate vertex location to hull origin
		for(v=X_; v<=Z_; v++) {
			DipLoc[v] = Hull->vertex[i].p[v] - Hull->Vo[v];
			DipOri[v] = Hull->vertex[i].v[v];
		}

		// compute basis function for vertex
		GetBasis(DipLoc, Hull->scale, basis, gradbasis);
		for(j=1; j<=BASIS; j++)
			for(v=X_, gradbasisn[j]=0.; v<=Z_; v++)		// gradbasisn = gradbasis <dot> vertex orientation
				gradbasisn[j] += gradbasis[j][v] * DipOri[v];

		// accumulate C'C upper triangle
		for(j=1; j<=BASIS; j++)
			for(k=1; k<=j; k++)
				ctc[j][k] += gradbasisn[j] * gradbasisn[k];

		// accumulate C'b over all detection locations
		for(j=1; j<=M; j++) {

			// translate detector location to hull origin
			for(v=X_; v<=Z_; v++) {
				SensLoc[v] = detector[j].Vi.p[v] - Hull->Vo[v];
				SensOri[v] = detector[j].Vi.v[v];
			}

			// compute forward solution for dipole in spherical conductor
			xbd(DipLoc, SensLoc, Bd);					// solve Sarvas forward solution for vertex & detector locations
			for(v=X_, x=0.; v<=Z_; v++)					// x is dot product of vertex normal field with detector orientation
				for(w=X_; w<=Z_; w++)
					x += Bd[v][w] * DipOri[v] * SensOri[w];
	
			// C'b
			for(k=1; k<=BASIS; k++)
				ctb[k][j] += x * gradbasisn[k];
		}
	}

	// mirror C'C to lower triangle
	for(j=1; j<=BASIS; j++)
		for(k=j+1; k<=BASIS; k++)
			ctc[j][k] = ctc[k][j];

	// compute coefficients a = [C'C]^-1 [C'b]
	xludcmp(ctc, indx, &d);
	for(j=1; j<=M; j++) {
		for(i=1; i<=BASIS; i++)
			achan[i] = ctb[i][j];
		xlubksb(ctc, indx, achan);
		for(i=1; i<=BASIS; i++)
			coeffs[i][j] = achan[i];
	}
} 
