// NLeadField() - compute the M x 3N lead field matrix
//
//	Author:	Stephen E. Robinson
//		MEG Core Facility
//		NIMH
//

#include <lfulib.h>


void	NLeadField(
	VOXELINFO	*Voxel,				// Voxel[N] -- array of N voxel structures
	double		**L,				// L[M][3N] -- lead field matrix
	HeaderInfo	*Header,			// header information with sensor list
	ChannelInfo	*Channel,			// sensor information
	HULL		*Hull,				// cortical hull model
	int			N					// number of sources
)
{
	static double	**B;			// B[M][3] -- lead field matrix for single source
	double			Vc[3];			// cartesian position vector
	register int	i;				// i-index
	register int	j;				// j-index
	register int	k;				// k-index
	register int	m;				// primary sensor index
	int				v;				// vector index
	static int		M = 0;			// number of primary sensors

	// if req'd, allocate memory & invert covariance
	if(M == 0) {
		M = Header->NumPri;			// make it so, Number One...

		// allocate matrices & arrays
		if((B = (double **)malloc((size_t)M * sizeof(double *))) == NULL)
			allocfailed("B[]");
		for(i=0; i<M; i++)
			if((B[i] = (double *)malloc((size_t)3 * sizeof(double))) == NULL)
				allocfailed("B[][]");
	}

	// compute ECD lead field matrix - compute forward solution matrix for unit dipole terms
	for(i=0; i<N; i++) {
		for(v=X_; v<=Z_; v++)
			Vc[v] = Voxel[i].p[v];
#if ELEKTA
		ECDLeadFieldElekta(Vc, Header, Channel, Hull, B);
#else
		ECDLeadField(Vc, Header, Channel, Hull, B);
#endif
		for(j=0, k=3*i; j<3; j++, k++)	// copy B[M][3] to N-source matrix L[M][QV]
			for(m=0; m<M; m++)
				L[m][k] = B[m][j];
	}
}
