// DATAtoSymbolArray() -- computes symbols from rank vectors
//	& outputs symbol time series
//
//	arguments to subroutine:
//		x			-- data time-series (input)
//		Sym5		-- symbol time-series N=5 (output)
//		T			-- number of samples in x (input)
//		TE			-- number of symbols derived from time-series (output)
//		lowpass		-- data lowpass frequency (input)
//		rate		-- sample rate (input)
//
//	Author:
//		Stephen E. Robinson, PhD
//		MEG Core Group
//		NIMH
//

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <rvelib.h>

void	Data2SymbolArray(
	double	*x,						// x[T] -- measurement time-series (input)
	short	*Symbol,				// Symbol[TE] -- symbol array (output)
	int		T,						// number of samples in x (input)
	int		*TE,					// number of symbols computed (output)
	int		E,						// phase space dimension (length of vector) (input)
	double	LPFreq,					// lowpass frequency (Hz) (input)
	double	rate					// sample rate (Hz) (input)
) {
	double			vector[7];		// vector of lagged measurement values (up to 7)
	static int		D = 0;			// stride (lags)
	int				w;				// word index
	int				t;				// time index
	int				tt;				// alternate time index

	// sanity check
	if(E < 3 || E > 6)
		cleanup("phase space dimension out of range");

	// one-time initialization of constants
	if(D == 0)
		D = (int)floor(rate / (2. * LPFreq));	// the lags depend upon the sample rate & upper frequency limit
	*TE = T - ((E - 1) * D);					// the output time-series is truncated by the lags

	// for the specified segment of data x(t)...
	for(t=0; t<*TE; t++) {

		// extract measurement vector from x & convert to symbol
		for(w=0; w<E; w++) {
			tt = t + w * D;
			vector[w] = x[tt];
		}
		Symbol[t] = Vec2Symbol(vector, E);
	}
}
