// RandomSurrogate() -- generates an array of random integers from 0 to max
//
//	Author:	Stephen E. Robinson
//			MEG Core Group
//			NIMH
//

#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <sys/time.h>
#include <rvelib.h>

#define TRUE	1
#define FALSE	0


void	RandomSurrogate(
	short	*SymPhsX,			// SymPhsX[T] -- x random phase-space symbols
	short	*SymAdvX,			// SymAdvX[T] -- x random advance state symbols
	short	*SymPhsY,			// SymPhsY[T] -- y random phase-space symbols
	short	*SymAdvY,			// SymAdvY[T] -- y random advance state symbols
	int		E,					// phase space dimension
	int		T					// number of samples
) {
	int			t;				// sample index
	static int	N;				// maximum number of advance symbols
	static int	S;				// maximum number of phase space symbols
	time_t		secs;			// system time
	static int	init = FALSE;	// initialization flag

	// on each call, initialize seed using system time
	if(init == FALSE) {
		time(&secs);
		srandom((unsigned int)secs);
		switch(E) {
			case 3: S = 6; N = 4; break;
			case 4: S = 24; N = 5; break;
			case 5: S = 120; N = 6; break;
			case 6: S = 720; N = 7; break;
			default: cleanup("embedding space dimension out of range"); break;
		}
		init = TRUE;
	}
	
	// fill arrays with random integers
	for(t=0; t<T; t++) {
		SymPhsX[t] = (short)(random() % S);
		SymAdvX[t] = (short)(random() % N);
		SymPhsY[t] = (short)(random() % S);
		SymAdvY[t] = (short)(random() % N);
	}
}
