// Vec2Advance() ranks a length E+1 measurement vector &
//	returns the rank of the Eth symbol for the advance.
//	This routine works for E=3..6, only! Enter with an
//	E+1 length measurement vector. Exit with the rank
//	value of the (E-1)th (last) symbol. The range of the
//	advance symbol value is 0 to E-1
//
//	Author:	Stephen E. Robinson
//			MEG Core Facility
//			NIMH
//

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <rvelib.h>


short	Vec2Advance(
	double	*x,						// x[E+1] -- measurement vector
	int		E						// this is the phase-space dimension -- not the vector length
) {
	double			max;			// maximum value
	static short	Rank[7];		// rank vector for x or y (large enough for E=6)
	static int		W;				// augmented vector size
	int				w;				// word index
	int				index = 0;		// rank index
	int				rank;			// sample rank

	// select definitions from phase space dimension
	switch(E) {
		case 3: W = 4; break;
		case 4: W = 5; break;
		case 5: W = 6; break;
		case 6: W = 7; break;
		default: cleanup("embedding space dimension out of range"); break;
	}

	// initialize rank values for augmented vector W
	for(w=0; w<W; w++)
		Rank[w] = -1;

	// rank measurement vector (largest is rank 0)
	for(rank=0; rank<W; rank++) {
		for(w=0, max=(-HUGE); w<W; w++)
			if(Rank[w] == -1 && x[w] > max) {
				max = x[w];
				index = w;
			}
		Rank[index] = rank;
	}
	return(Rank[W-1]);
}
