// Vec2Symbol() -- computes symbolic from phase-space measurement vector
//  Works for E=3..6
//
//  Author:
//      Stephen E. Robinson, PhD
//      MEG Core Group
//      NIMH
//

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <siglib.h>
#include <rvelib.h>
#include <SymbolTables.h>

#define TRUE    1
#define FALSE   0

short   Vec2Symbol(
    double  *x,                     // x[E] -- measurement vector
    int     E                       // phase space dimension
) {
    double          max;            // maximum value
    static int      *Symbol;        // symbols pointer
    static int      Rank[MAX_E];    // rank vector for x or y (large enough for E=6)
    static int      Redux;          // reduction
    static int      S;              // number of possible symbols
    static int      W;              // length of rank vector
    int             s;              // states index
    int             v;              // value index
    int             w;              // word index
    int             value;          // integer value
    int             index;          // rank index
    int             rank;           // sample rank

    // select definitions from phase space dimension
    switch(E) {
        case 3:     // 3-dimensional phase space
            S = 6;
            W = 3;
            Redux = 100;
            Symbol = Symbol3;
            break;
        case 4:     // 4-dimensional phase space
            S = 24;
            W = 4;
            Redux = 1000;
            Symbol = Symbol4;
            break;
        case 5:     // 5-dimensional phase space
            S = 120;
            W = 5;
            Redux = 10000;
            Symbol = Symbol5;
            break;
        case 6:     // 6-dimensional phase space
            S = 720;
            W = 6;
            Redux = 100000;
            Symbol = Symbol6;
        default:
            cleanup("embedding space dimension out of range");
            break;
    };

    // initialize rank values
    for(w=0; w<W; w++)
        Rank[w] = -1;

    // rank the measurement vector (largest is rank 0)
    for(rank=0; rank<W; rank++) {
        for(w=index=0, max=(-HUGE); w<W; w++)
            if(Rank[w] == -1 && x[w] > max) {
                max = x[w];
                index = w;
            }
        Rank[index] = rank;
    }

    // convert rank vector of x to base10 integer
    for(w=0, v=Redux, value=0; w<W; w++) {
        value += Rank[w] * v;
        v /= 10;
    }

    // scan symbol table for rank match
    for(s=0; s<S; s++)
        if(Symbol[s] == value) {
            return((short)s);
        }

    fprintf(stderr, "[Vec2Symbol] failed to find symbol in table\n");
    abort();
}
