// randsample() uses the Fisher-Yates shuffle to randomize the
//	order of samples in a time-series. It is a simple, elegant
//	algorithm that is useful for making surrogate time-series
//	data. It is intended for testing RVE and other measures of
//	non-linear dynamics.
//
//	Author:	Stephen E. Robinson
//			MEG Core Group
//			NIMH
//

#include <stdlib.h>
// #include <stdio.h>
#include <math.h>
#include <time.h>
#include <sys/time.h>
#include <rvelib.h>

void	randsample(
	double	*In,			// input time-series
	double	*Out,			// output randomized time-series
	int		T				// number of samples
)

{
	double		tmp;		// for sample exchange
	int			i;
	int			j;
	static int	N = 0;

	// on first call, initialize seed using system time
	if(N == 0) {
		srandom((time_t)time((time_t *)0));
		N = T;
	}

	// Fisher-Yates shuffle of elements
	for(i=0; i<N; i++)			// copy input time series to output (so input is not overwritten)
		Out[i] = In[i];
	for(i=(N-1); i>0; i--) {	// rearrange samples in random order
		j = (int)floorl(((long double)i - 0.5) * (long double)random() / (long double)RAND_MAX);
		if(i == j)
			cleanup("error in shuffle");
		tmp = Out[i];			// save the ith sample value
		Out[i] = Out[j];		// copy the jth sample (randomly selected between 1 & i) to the ith position
		Out[j] = tmp;			// copy the ith sample value to the jth position
	}
}
