// DIPLeadField3D() -- compute lead field matrix L for 3 dipoles
//	for an array of sensors.
//
//	Author:	Stephen E. Robinson (after Nolte, Sarvas, & Hamalainen)
//		MEG Core Facility
//		NIMH
//

#include <stdlib.h>
#include <samlib.h>
#include <math.h>
#include <cme.h>
#include <model.h>


int	DIPLeadField3D(
	double			Org[3],			// dipole origin
	double			**L,			// L[3][M] - 3 x M lead field matrix (external array allocation)
	HeaderInfo		*Header,		// header information, including primary & reference sensor indices
	ChannelInfo		*Channel,		// Channel[M] -- channel structure with primary and reference sensors
	int				Order			// integration order: 1, 2, 3, 4, 5, & 7 are implemented
) {
	FIELD			Q6;				// test dipole

	// set dipole origin for all dipole terms
	Q6.p[X_] = Org[X_];	Q6.p[Y_] = Org[Y_];	Q6.p[Z_] = Org[Z_];

	// (1) compute Qx
	Q6.v[X_] = 1.;	Q6.v[Y_] = 0.;	Q6.v[Z_] = 0.;
	if(FwdVec(&Q6, L[Qx_], Header, Channel, Order) == -1)
		return -1;

	// (2) compute Qy
	Q6.v[X_] = 0.;	Q6.v[Y_] = 1.;	Q6.v[Z_] = 0.;
	if(FwdVec(&Q6, L[Qy_], Header, Channel, Order) == -1)
		return -1;

	// (3) compute Qz
	Q6.v[X_] = 0.;	Q6.v[Y_] = 0.;	Q6.v[Z_] = 1.;
	if(FwdVec(&Q6, L[Qz_], Header, Channel, Order) == -1)
		return -1;

	// normal return
	return 0;
}
