// FwdVec() -- compute forward solution vector for a gradiometer
//  (with references) due to current dipole in homogeneously
//  conducting sphere. This subroutine supports multiple local spheres.
//	This version features programmable integration order from 1st to 7th.
//
//	Author:	Stephen E. Robinson
//		MEG Core Facility
//		NIMH
//

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <geoms.h>
#include <samlib.h>

int	FwdVec(
	FIELD		*Q6,			// location & orientation of current dipole
	double		*Bp,			// Bp[M] -- forward solution vector
	HeaderInfo	*Header,		// header information, including primary & reference sensor indices
	ChannelInfo	*Channel,		// Channel[M] -- channel structure with primary and reference sensors
	int			Order			// integration order: 1, 2, 3, 4, 5, & 7 are implemented
)

{
	static double	*Br;		// Br[N] -- reference sensor field array
	int				i;			// index
	int				j;			// another index
	int				k;			// yet another index
	int				g;			// still going...
	int				found;		// flag for reference channel index search
	static int		R = 0;		// number of references

	// if req'd, allocate memory for reference sensor field
	if(R == 0) {
		R = Header->NumRef;
		if((Br = (double *)malloc((size_t)(R * sizeof(double)))) == NULL)
			return -1;
	}

	// compute reference channel forward solution for Target source
	for(i=0; i<R; i++) {
		j = Header->RsIndex[i];
		Br[i] = Fwd2(Q6, &Channel[j].Geom.MEGSensor, Order);
	}

	// compute primary channel (balanced) forward solution for Target source
	for(i=0; i<Header->NumPri; i++) {
		j = Header->PsIndex[i];
		Bp[i] = Fwd2(Q6, &Channel[j].Geom.MEGSensor, Order);
		for(g=0; g<Channel[j].NumBal; g++) {
			for(k=0, found=FALSE; k<Header->NumRef; k++)
				if(Channel[j].Balance[g].RefChan == Header->RsIndex[k]) {
					found = TRUE;
					break;
				}
			if(found == FALSE)
				cleanup("couldn't find reference channel index");
			Bp[i] -= Channel[j].Balance[g].Weight * Br[k];
		}
	}

	return 0;
}
