// Construct various pathnames.

#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <unistd.h>
#include "samutil.h"
#include "sam_param.h"

// Copy pattern, with replacements, into path.

// For example, a pattern of "%M/%P/%s" is the default.
// %M is p->MRIdirectory
// %P is setname[:p->NumPrefix]
// %s is the passed name
// %H is the hash code (first _ delimited field of setname)
// %S is the study (second _ field)
// %D is the date (third)
// %R is the run (fourth)

// path is output, using the pattern for the MRI directory
// if exist is set, the resulting path must exist

void GetMRIPath(char *path, int len, PARMINFO *p, char *name, int exist)
{
    int i;
    char *s, *t, *setname;
    char *f[4];     // H, S, D, and R pointers

    // If the name that's passed in is an absolute pathname, use that.

    if (name[0] == '/') {
        strncpy(path, name, len);
        return;
    }

    // Parse the setname, for %H, etc. Up to 4 underscore delimited fields.

    s = p->DataSetName;
    t = rindex(s, '/');     // just the setname
    if (t != NULL) {
        s = t + 1;
    }
    setname = s;

    t = new_string(strlen(s) + 1);
    f[0] = t;
    i = 1;
    while (*s) {
        if (*s == '_' && i < 4) {
            *t++ = '\0';
            f[i] = t;
            i++;
        } else {
            *t++ = *s;
        }
        s++;
    }
    *t = '\0';

    // Parse the pattern, output what it says.

    s = p->MRIPattern;
    t = path;
    while (*s) {
        if (*s != '%') {
            *t++ = *s++;
        } else {
            s++;
            if (*s == 'M') {
                t = strecpy(t, p->MRIdirectory);
            } else if (*s == 'P') {
                strncpy(t, setname, p->NumPrefix);
                t += p->NumPrefix;
                *t = '\0';
            } else if (*s == 's') {
                t = strecpy(t, name);
            } else if (*s == 'H') {
                t = strecpy(t, f[0]);
            } else if (*s == 'S') {
                t = strecpy(t, f[1]);
            } else if (*s == 'D') {
                t = strecpy(t, f[2]);
            } else if (*s == 'R') {
                t = strecpy(t, f[3]);
            }
            s++;
        }
        if (t > path + len) {   // we already borked it
            fatalerr("filename too long: %s", path);
        }
    }
    free(f[0]);

    if (exist) {
        if (access(path, F_OK) != 0) {
            fatalerr("can't access %s", path);
        }
    }
}
