// MoveFrame() -- Move an array of all sensors (primary & reference)
//	to a new frame. Used for generating beamformer weight from atlas
//	of voxels & normals.
//
//	This version uses a 4-parameter tranform: translation in x, y, z
//	& rotation around y-axis.
//
//      Author: Stephen E. Robinson
//              MEG Core Facility
//              NIMH
//

#include <geoms.h>
#include <samlib.h>
#include <siglib.h>


void	MoveFrame(
	HeaderInfo	*Header,			// dataset header info
	ChannelInfo	*InChannel,			// input channel array
	ChannelInfo	*OutChannel,		// output (transformed) channel array
	char		*TransFile			// full path name for transform file
)

{
	XFORM		Where;				// where to move all SQUID sensors
	double		To[3];				// translation (metres)
	double		Ry;					// rotation around y-axis (radians)
	int			*SQUIDIndex;		// SQUIDIndex[S] -- index of channels for rotation/translation
	int			S;					// total SQUID sensors
	int			i;					// channel index
	int			j;					// 'nuther index
	int			v;					// vector index
	FILE		*fp;				// generic file pointer

	// read transform file (translation is in millimetres & theta in milliradians)
	if((fp = fopen(TransFile, "r")) == NULL)
		cleanup("transform file not found");
	if(fscanf(fp, "%lf%lf%lf%lf", &To[X_], &To[Y_], &To[Z_], &Ry) != 4)
		cleanup("error reading transform values");
	fclose(fp);

	// convert transform to metres & radians
	for(v=X_; v<=Z_; v++)
		To[v] *= 0.001;
	Ry *= 0.001;

	// determine S
	for(i=S=0; i<Header->NumChannels; i++)
		if(InChannel[i].ChannelType == TYPE_MEG
		|| InChannel[i].ChannelType == TYPE_REF_GRAD
		|| InChannel[i].ChannelType == TYPE_REF_MAG)
			S++;

	// create SQUID index
	if((SQUIDIndex = (int *)malloc((size_t)S * sizeof(int))) == NULL)
		allocfailed("SQUIDIndex[]");
	for(i=j=0; i<Header->NumChannels; i++)
		if(InChannel[i].ChannelType == TYPE_MEG
		|| InChannel[i].ChannelType == TYPE_REF_GRAD
		|| InChannel[i].ChannelType == TYPE_REF_MAG) {
			SQUIDIndex[j] = i;
			j++;
		}

	// move sensors to specified To & Ro
	for(v=X_; v<=Z_; v++)
		Where.Translate[v] = To[v];
	YtoR(Ry, Where.Rotate);
	MoveMEG(InChannel, OutChannel, SQUIDIndex, S, &Where);
}

