// NLeadField2D() - compute the M x 2N lead field matrix
//  for the single local sphere model
//
//  Author: Stephen E. Robinson
//      MEG Core Facility
//      NIMH
//

#include <stdio.h>
#include <stdlib.h>
#include <samlib.h>
#include <siglib.h>
#include <lfulib.h>


void    NLeadField2D(
    VOXELINFO   *Voxel,             // Voxel[N] -- array of N voxel structurs
    double      **L,                // L[M][2N] -- lead field matrix
    HeaderInfo  *Header,            // header information with sensor list
    ChannelInfo *Channel,           // sensor information
    int         N                   // number of sources
)
{
    static double   **B;            // B[2][M] -- lead field matrix for single source
    double          Vc[3];          // cartesian position vector
    register int    i;              // i-index
    register int    j;              // j-index
    register int    k;              // k-index
    register int    m;              // primary sensor index
    int             v;              // vector index
    static int      M = 0;          // number of primary sensors

    // if req'd, allocate memory & invert covariance
    if(M == 0) {
        M = Header->NumPri;         // make it so, Number One...

        // allocate matrices & arrays
        if((B = (double **)malloc((size_t)2 * sizeof(double *))) == NULL)
            cleanup("'B[]' allocation failed");
        for(i=0; i<2; i++)
            if((B[i] = (double *)malloc((size_t)M * sizeof(double))) == NULL)
                cleanup("'B[][]' allocation failed");
    }

    // compute ECD lead field matrix - compute forward solution matrix for unit dipole terms
    for(i=0; i<N; i++) {
        for(v=X_; v<=Z_; v++)
            Vc[v] = Voxel[i].p[v];
        if(DIPLeadField2D(Vc, B, Header, Channel, 2) == -1)
            cleanup("'DIPLeadField2D()' failed");
        for(j=0, k=2*i; j<2; j++, k++)  // copy B[2][M] to N-source matrix L[M][2N]
            for(m=0; m<M; m++)
                L[m][k] = B[j][m];
    }
}
