// NLeadField3D() - compute the M x 3N lead field matrix
//	for the multiple local sphere model
//
//	Author:	Stephen E. Robinson
//		MEG Core Facility
//		NIMH
//

#include <stdio.h>
#include <stdlib.h>
#include <samlib.h>
#include <siglib.h>
#include <lfulib.h>


void	NLeadField3D(
	VOXELINFO	*Voxel,				// Voxel[N] -- array of N voxel structurs
	double		**L,				// L[M][3N] -- lead field matrix
	HeaderInfo	*Header,			// header information with sensor list
	ChannelInfo	*Channel,			// sensor information
	int			N					// number of sources
)
{
	static double	**B;			// B[3][M] -- lead field matrix for single source
	double			Vc[3];			// cartesian position vector
	register int	i;				// i-index
	register int	j;				// j-index
	register int	k;				// k-index
	register int	m;				// primary sensor index
	int				v;				// vector index
	static int		M = 0;			// number of primary sensors

	// if req'd, allocate memory & invert covariance
	if(M == 0) {
		M = Header->NumPri;			// make it so, Number One...

		// allocate matrices & arrays
		if((B = (double **)malloc((size_t)3 * sizeof(double *))) == NULL)
			cleanup("'B[]' allocation failed");
		for(i=0; i<3; i++)
			if((B[i] = (double *)malloc((size_t)M * sizeof(double))) == NULL)
				cleanup("'B[][]' allocation failed");
	}

	// compute ECD lead field matrix - compute forward solution matrix for unit dipole terms
	for(i=0; i<N; i++) {
		for(v=X_; v<=Z_; v++)
			Vc[v] = Voxel[i].p[v];
		if(DIPLeadField3D(Vc, B, Header, Channel, 2) == -1)
			cleanup("'DIPLeadField3D()' failed");
		for(j=0, k=3*i; j<3; j++, k++)	// copy B[3][M] to N-source matrix L[M][3N]
			for(m=0; m<M; m++)
				L[m][k] = B[j][m];
	}
}
