// PutCov() -- write covariance file (with header) to disk
//
//	Author:	Stephen E. Robinson
//		MEG Core Facility
//		NIMH
//

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <SAMfiles.h>
#include <samlib.h>
#include <version.h>

void	PutCov(
	char	*CovName,		// fully-qualified path name
	COV_HDR	CovHeader,		// the filled-in header
	int		*ChanIndex,		// channel index (base-0)
	double	**Covariance	// Covariance[M][M]
)

{
	COV_HDR			WriteHeader;	// copy of covariance header to write
	int				i;				// channel index
	int				j;				// alternate channel index
	static int32_t	*WriteIndex;	// copy of channel index array to write
	static int	M = 0;				// number of primary sensors used
	char		Identity[16] =		// covariance identity string
		"SAMCOVAR";
	static FILE	*fout;				// output file pointer

	// if req'd, allocate array for indexing channels
	if(CovHeader.NumChans != M) {
		if(CovHeader.NumChans > M)
			free((void *)WriteIndex);
		if((WriteIndex = (int32_t *)malloc((size_t)(CovHeader.NumChans * sizeof(int32_t)))) == NULL)
			allocfailed("WriteIndex[]");
		M = CovHeader.NumChans;
	}

	// open file for write
	if((fout = fopen(CovName, "w")) == NULL)
		cleanup("can't open covariance file for write");
	if(fwrite((void *)Identity, 8, 1, fout) != 1)
		cleanup("covariance ID string write failed");
	WriteHeader.Version = SAM_REV;			// actual software revision number
	WriteHeader.NumChans = CovHeader.NumChans;
	WriteHeader.HPFreq = CovHeader.HPFreq;
	WriteHeader.LPFreq = CovHeader.LPFreq;
	WriteHeader.BWFreq = CovHeader.BWFreq;
	WriteHeader.Noise = CovHeader.Noise;
	WriteHeader.NumSegments = CovHeader.NumSegments;
	WriteHeader.NumSamples = CovHeader.NumSamples;
	WriteHeader.CovType = CovHeader.CovType;
	WriteHeader.dummy1 = 0;
	if(fwrite((void *)&WriteHeader, sizeof(COV_HDR), 1, fout) != 1)
		cleanup("covariance header write failed");
	for(i=0; i<M; i++)
		WriteIndex[i] = ChanIndex[i];
	if(fwrite((void *)WriteIndex, sizeof(int32_t), M, fout) != M)
		cleanup("covariance index write failed");
	for(i=0; i<M; i++)
		for(j=0; j<M; j++)
			if(fwrite((void *)&Covariance[i][j], sizeof(double), 1, fout) != 1)
				cleanup("covariance matrix write failed");
	fclose(fout);
}
