// cholinv() -- invert a positive-definite matrix
//  using Cholesky decomposition
//
//  Shamelessly "borrowed" from "Numerical Recipes in C"
//
//  Author: Stephen E. Robinson
//          MEG Core Facility
//          NIMH
//

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "samutil.h"

void    Cholesky(
    double  **C,                // C[M][M] -- positive definite matrix (input)
    double  **A,                // A[N][N] -- Cholesky lower diagonal (output)
    int     M)                  // matrix rank
{
    static double   *p;         // p[N] -- diagonal vector
    register double sum;
    int             i;
    int             j;
    int             k;
    static int      rank = 0;   // remember matrix rank

    // allocate diagonal vector
    if(rank != M) {
        if(rank != 0)
            free((void *)p);
        if((p = (double *)malloc((size_t)(M * sizeof(double)))) == NULL)
            allocfailed("p[]");
        rank = M;
    }

    // copy C to A
    for(i=0; i<M; i++)
        for(j=0; j<M; j++)
            A[i][j] = C[i][j];

    // Cholesky decomposition
    for(i=0; i<M; i++) {
        for(j=i; j<M; j++) {
            for(k=i-1, sum=A[i][j]; k>=0; k--)
                sum -= A[i][k] * A[j][k];
            if(i == j) {
                if(sum <= 0.)
                    cleanup("Cholesky decomposition failed");
                p[i] = sqrt(sum);
            } else
                A[j][i] = sum / p[i];
        }
    }
}
