// GenEigSys22() -- generalized eigenvalue problem for 2x2 matrices
//
//	Author:	Stephen E. Robinson
//			Neuromagnetism Laboratory
//			Dept. of Neurology
//			Henry Ford Hospital
//			Detroit, MI
//			Copyright (c) 2007
//

#include <math.h>
#include <siglib.h>

void	geig22(
	double	A[2][2],		// 2x2 matrix A
	double	B[2][2],		// 2x2 matrix B
	double	Vec[2][2],		// 2x2 eigenvector matrix
	double	Val[2]			// eigenvalues
)

{
	double	a11, a11_2;
	double	a12, a12_2;
	double	a22, a22_2;
	double	b11, b11_2;
	double	b12, b12_2;
	double	b22, b22_2;
	double	u1, u1_2;
	double	u2, u2_2;
	double	u3;
	double	u4;
	double	u5;
	double	u6;
	double	u7;
	double	u8;
	double	u9;
	double	u10;
	double	l1;
	double	l2;
	double	k1;
	double	k2;

	a11 = A[0][0];
	a12 = A[0][1];
	a22 = A[1][1];
	b11 = B[0][0];
	b12 = B[0][1];
	b22 = B[1][1];
	u1 = a12 * b11 - a11 * b12;
	u2 = a22 * b11 - 2. * a12 * b12 + a11 * b22;
	a11_2 = a11 * a11;
	a12_2 = a12 * a12;
	a22_2 = a22 * a22;
	b11_2 = b11 * b11;
	b12_2 = b12 * b12;
	b22_2 = b22 * b22;
	u1_2 = u1 * u1;
	u2_2 = u2 * u2;
	u3 = sqrt(-4. * (a12_2 - a11 * a22) * (b12_2 - b11 * b22) + u2_2);
	u4 = a22_2 * b11_2
		+ 2. * a12_2 * b11 * (b11 + b22)
		- 2. * a12 * b12 * (a22 * b11 + a11 *(2. * b11 + b22))
		+ a22 * (2. * a11 * b12_2 - 2. * a11 * b11 * b22 + b11 * u3)
		+ a11 * (2. * a11 * b12_2 + a11 * b22_2 - b22 * u3);
	u5 = a22_2 * b11_2
		+ 2. * a11_2 * b12_2
		+ 2. * a12_2 * b11 * (b11 + b22) - 2. * a12 * b12 * (a22 * b11 + a11 * (2. * b11 + b22))
		+ a11 * b22 *(a11 * b22 + u3)
		+ a22 * (2. * a11 * (b12_2 - b11 * b22) - b11 * u3);
	u6 = a22 * (-2. * b12_2 + b11 * b22) + b22 * (2. * a12 * b12 - a11 * b22 + u3);
	u7 = a22 * (-2. * b12_2 + b11 * b22) + b22 * (2. * a12 * b12 - a11 * b22 - u3);
	u8 = a22 * b11 * b12 - 2. * a12 * b11 * b22 + b12 * (a11 * b22 - u3);
	u9 = a22 * b11 * b12 - 2. * a12 * b11 * b22 + b12 * (a11 * b22 + u3);
	u10 = 2. * (b12_2 - b11 * b22);
	l1 = (u3 - u2) / u10;
	l2 = (-u3 - u2) / u10;
	k1 = M_SQRT2 * sqrt(u1_2 / u4);
	k2 = M_SQRT2 * sqrt(u1_2 / u5);
	if(l1 > l2) {
		Val[0] = l1;
		Val[1] = l2;
		Vec[0][0] = k1 * (u6 / u8);
		Vec[0][1] = k1;
		Vec[1][0] = k2 * (u7 / u9);
		Vec[1][1] = k2;
	} else {
		Val[0] = l2;
		Val[1] = l1;
		Vec[0][0] = k2 * (u7 / u9);
		Vec[0][1] = k2;
		Vec[1][0] = k1 * (u6 / u8);
		Vec[1][1] = k1;
	}
}
