// invert() - matrix inversion by method of Gauss-Jordan.
//  Shamelessly translated from Pascal version found in:
//  Miller, "Pascal Programs for Scientists & Engineers", p. 87.
//
//  Author: Stephen E. Robinson
//          MEG Core Facility
//          NIMH
//

#include <math.h>
#include <stdlib.h>
#include <siglib.h>
#include <lfulib.h>

void invert(
    double  **b,        // matrix to be inverted (also result)
    int     N           // size of row or column
) {
    double          pivot;
    double          t;
    double          big;
    int             row = 0;
    int             col = 0;
    int             ll;
    register int    i;
    register int    j;
    register int    k;
    register int    l;
    static int      *pivotIndex;
    static int      *rowIndex;
    static int      *columnIndex;
    static int      mem = 0;

    if(mem != N) {
        if(mem != 0) {
            free((void *)pivotIndex);
            free((void *)rowIndex);
            free((void *)columnIndex);
        }
        if((pivotIndex = (int *)malloc((size_t)N * sizeof(int))) == NULL)
            allocfailed("pivotIndex[]");
        if((rowIndex = (int *)malloc((size_t)N * sizeof(int))) == NULL)
            allocfailed("rowIndex[]");
        if((columnIndex = (int *)malloc((size_t)N * sizeof(int))) == NULL)
            allocfailed("columnIndex[]");
        mem = N;
    }

    for(j=0; j<N; j++)
        pivotIndex[j] = 0;
    for(i=0; i<N; i++) {
        big = 0.;
        for(j=0; j<N; j++)
            if(pivotIndex[j] != 1)
                for(k=0; k<N; k++) {
                    if(pivotIndex[k] == 0) {
                        if(fabs(b[j][k]) >= big) {
                            big = fabs(b[j][k]);
                            row = j;
                            col = k;
                        }
                    } else {
                        if(pivotIndex[k] > 1)
                            cleanup("invert failed");
                    }
                }
        pivotIndex[col]++;
        if(row != col)
            for(l=0; l<N; l++)
                dswap(&b[row][l], &b[col][l]);
        rowIndex[i] = row;
        columnIndex[i] = col;
        if(b[col][col] == 0.)
            cleanup("invert failed");
        pivot = b[col][col];
        b[col][col] = 1.;
        for(l=0; l<N; l++)
            b[col][l] /= pivot;
        for(ll=0; ll<N; ll++)
            if(ll != col) {
                t = b[ll][col];
                b[ll][col] = 0.;
                for(l=0; l<N; l++)
                    b[ll][l] -= b[col][l] * t;
            }
    }
    for(l=(N-1); l>=0; l--) {
        if(rowIndex[l] != columnIndex[l]) {
            row = rowIndex[l];
            col = columnIndex[l];
            for(k=0; k<N; k++)
                dswap(&b[k][row], &b[k][col]);
        }
    }
}
