/* File utilities. */

#include "samutil.h"
#include <unistd.h>

/* Open a file or exit on failure. */

FILE *fileopen(char *name, char *mode)
{
    FILE *f;

    if ((f = fopen(name, mode)) == NULL) {
        fatalerr("can't %s '%s'", *mode == 'r' ? "open" : "write", name);
    }
    return f;
}

/* Check to see if a file exists before we open it. */

int fileexists(char *name)
{
    if (access(name, R_OK) < 0) {
        return FALSE;
    }
    return TRUE;
}

/* Like fgets() but remove the trailing newline. Return a pointer to
the nul at the end of the string or NULL on error. */

char *fgetline(char *buf, int maxlen, FILE *infile)
{
    int i, c;
    char *s;

    s = buf;
    i = maxlen - 1;
    if (i < 0) {
        return NULL;
    }
    if (i == 0) {
        *s = '\0';
        return s;
    }
    c = getc(infile);
    if (c == EOF) {
        return NULL;
    }
    while (c != '\n' && c != EOF) {
        *s++ = c;
        if (--i <= 0) {
            break;
        }
        c = getc(infile);
    }
    *s = '\0';
    return s;
}
