#! /usr/bin/python

import sys, getopt
import numpy as np
import nibabel
##import h5py
from scipy.spatial import KDTree

Step = 3
Radius = 2.

optlist, args = getopt.getopt(sys.argv[1:], "r:s:")
for opt, arg in optlist:
    if opt == '-s':
	Step = int(arg)
    elif opt == '-r':
	Radius = float(arg)

mri = args[0]
surf = args[1]

# Load the MRI

im = nibabel.load(mri)
data = im.get_data()
toLPI = im.get_affine()
fromLPI = np.linalg.inv(toLPI)

# Load the surface verts and normals, convert from PRI m to LPI mm

##hfile = h5py.File(surf, 'r')
##surf = hfile['surface'][:]

surf = np.loadtxt(surf)

for i in range(len(surf)):
    v = surf[i][:3] * 1000.
    n = surf[i][3:]
    v = -v[1], v[0], v[2]
    n = -n[1], n[0], n[2]
    surf[i][:3] = v
    surf[i][3:] = n

k = KDTree(surf[:,:3])

Nx, Ny, Nz = data.shape
Rx = range(0, Nx, Step)
Ry = range(0, Ny, Step)
Rz = range(0, Nz, Step)

out = []
for x in Rx:
    for y in Ry:
	for z in Rz:
	    if data[x, y, z]:
		pos = [x, y, z, 1]
		a = np.dot(toLPI, pos)
		idx = k.query_ball_point(a[:3], Radius)
		if len(idx) > 0:
		    n = surf[idx][:,3:].mean(axis=0)
		    v = a[:3] / 1000.
		    l = [v[1], -v[0], v[2], n[1], -n[0], n[2]]
		    l = ' '.join(map(str, l))
		    out.append(l)

print len(out)
for l in out:
    print l
