#include <Python.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <arrayobject.h>

#include <unistd.h>
#include <float.h>
#include <strings.h>
#include <fcntl.h>
#include <model.h>
#include <samlib.h>
#include <siglib.h>
#include <lfulib.h>
#include <DataFiles.h>
#include <SAMfiles.h>
#include <samUtils.h>

static char *DSName;            /* there can be only one */
static HeaderInfo Header;
static ChannelInfo *Channel;
static EpochInfo *Epoch;
static HULL Hull;
extern int CoeffFlag;
static int M;
static double **L;

/* Python wrapper code. */

static char Doc_dsopen[] = "dsopen(filename)\n\
Open a CTF dataset.\n";

static PyObject *dsopen_wrap(PyObject *self, PyObject *args)
{
	char *dsname, *s;
	int i;

	if (!PyArg_ParseTuple(args, "s", &dsname)) {
		return NULL;
	}

	DSName = (char *)malloc(strlen(dsname) + 1);
	strcpy(DSName, dsname);

	fprintf(stderr, "opening %s\n", DSName);

	if (GetDsInfo(DSName, &Header, &Channel, &Epoch, NULL, TRUE) < 0) {
		fprintf(stderr, "failed to open dataset\n");
		return NULL;
	}

	/* read the hull.shape file */

	i = strlen(Header.DsPath) + 2;
	i += strlen(Header.SetName) + 15;
	s = (char *)malloc(i);
	strcpy(s, Header.DsPath);
	strcat(s, "/");
	strcat(s, Header.SetName);
	strcat(s, ".ds/hull.shape");
	GetHull(s, &Hull);
	CoeffFlag = TRUE;
	free(s);

	/* allocate the leadfield array */

	M = Header.NumPri;
	L = (double **)malloc(M * sizeof(double *));
	if (L == NULL) return NULL;
	for (i = 0; i < M; i++) {
		L[i] = (double *)malloc(3 * sizeof(double));
		if (L[i] == NULL) return NULL;
	}

	Py_INCREF(Py_None);
	return Py_None;
}

static char Doc_doFwd[] = "doFwd(pos, ori)\n\
pos and ori are three-tuples specifying position in cm and orientation,\n\
respectively.\n";

static PyObject *doFwd_wrap(PyObject *self, PyObject *args)
{
	int i, j;
	double pos[3], ori[3], d, *b;
	npy_intp dim[1];
	PyArrayObject *r;

	if (!PyArg_ParseTuple(args, "(ddd)(ddd)", pos, pos + 1, pos + 2, ori, ori + 1, ori + 2)) {
		return NULL;
	}

	/* Convert to meters. */

	for (j = 0; j < 3; j++) {
		pos[j] *= .01;
	}

	/* Normalize the orientation vector. */

	d = 0.;
	for (j = 0; j < 3; j++) {
		d += ori[j] * ori[j];
	}
	d = sqrt(d);
	for (j = 0; j < 3; j++) {
		ori[j] /= d;
	}

	/* Calculate the lead field, L, and dot it with the orientation. */

	ECDLeadField(pos, &Header, Channel, &Hull, L);

	dim[0] = M;
	r = (PyArrayObject *)PyArray_SimpleNew(1, dim, NPY_DOUBLE);
	if (r == NULL) {
		return NULL;
	}
	b = (double *)PyArray_DATA(r);

	for (i = 0; i < M; i++) {
		d = 0.;
		for (j = 0; j < 3; j++) {
			d += ori[j] * L[i][j];
		}
		b[i] = d;
	}

	return PyArray_Return(r);
}

static PyMethodDef Methods[] = {
	{ "dsopen", dsopen_wrap, METH_VARARGS, Doc_dsopen },
	{ "doFwd", doFwd_wrap, METH_VARARGS, Doc_doFwd },
	{ NULL, NULL, 0, NULL }
};

void initnolteFwd()
{
	Py_InitModule("nolteFwd", Methods);
	import_array();
}
