#include <Python.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <arrayobject.h>

extern void MovingPermEntropy(double *, double *, int, int, int, double, double);

/* Python wrapper code. */

static char Doc_perment[] = "MovingPermEntropy(x, permsize, tau, srate)\n\
Return the normalized moving permutation entropy of the 1D array x,\n\
given the integer permutation window size permsize,\n\
a real decay constant tau,\n\
and the real sampling rate srate in Hz.\n";

static PyObject *perment_wrap(PyObject *self, PyObject *args)
{
	int n;
	int permsize;
	double tau, srate;
	npy_intp dim[1];
	PyObject *o;
	PyArrayObject *a, *r;

	if (!PyArg_ParseTuple(args, "Oidd", &o, &permsize, &tau, &srate)) {
		return NULL;
	}

	a = (PyArrayObject *)PyArray_ContiguousFromObject(o, PyArray_DOUBLE, 1, 1);
	if (a == NULL) {
		return NULL;
	}
	n = a->dimensions[0];

	if (n <= permsize || permsize > 24) {
		Py_DECREF(a);
		return NULL;
	}

	dim[0] = n - permsize - 1;
	r = (PyArrayObject *)PyArray_SimpleNew(1, dim, PyArray_DOUBLE);
	if (r == NULL) {
		Py_DECREF(a);
		return NULL;
	}

	MovingPermEntropy((double *)a->data, (double *)r->data, 0, n-1, permsize, tau, srate);

	Py_DECREF(a);
	return PyArray_Return(r);
}

static PyMethodDef Methods[] = {
	{ "MovingPermEntropy", perment_wrap, METH_VARARGS, Doc_perment },
	{ NULL, NULL, 0, NULL }
};

void initperment()
{
	Py_InitModule("perment", Methods);
	import_array();
}
