#! /usr/bin/env python

import os, sys

surf = "smoothwm"                   # default surface to use

valid_s = ["orig", "pial", "smoothwm", "white"]
surfmsg = "surface must be one of " + ', '.join(valid_s) + " (default %s)" % surf

if len(sys.argv) < 2:
    print >> sys.stderr, "usage: FSnormals subject [surface]"
    print >> sys.stderr, surfmsg
    sys.exit(1)

subj = sys.argv[1]

if len(sys.argv) > 2:
    surf = sys.argv[2]

if surf not in valid_s:
    print >> sys.stderr, surfmsg
    sys.exit(1)

try:
    FShome = os.environ['FREESURFER_HOME']
except KeyError:
    print >> sys.stderr, "You must set the FREESURFER_HOME environment variable."
    sys.exit(1)

# We need the origin offset of the original MRI ...

dir = os.path.join(FShome, "subjects", subj, "mri", "orig")
try:
    os.chdir(dir)
except OSError:
    print >> sys.stderr, "Error locating subject directory %s" % dir
    sys.exit(1)

try:
    cmd = os.path.join(FShome, "bin", "mri_info")
    os.system("%s --cras 001.mgz > info.txt" % cmd)
    l = open("info.txt").readline().split()
    o = map(float, l)
except:
    print >> sys.stderr, "Can't find origin offset!"
    sys.exit(1)

# Now switch to the dir with the surfaces in it.

dir = os.path.join(FShome, "subjects", subj, "surf")
try:
    os.chdir(dir)
except OSError:
    print >> sys.stderr, "Error locating subject directory %s" % dir
    sys.exit(1)

# Generate the .asc files for both hemispheres.

print >> sys.stderr, "Generating %s surface meshes ..." % surf
cmd = os.path.join(FShome, "bin", "mris_convert")
t = (cmd, surf, surf)
os.system("%s rh.%s rh.%s.asc" % t)
os.system("%s -n rh.%s rh.%s.normals.asc" % t)
os.system("%s lh.%s lh.%s.asc" % t)
os.system("%s -n lh.%s lh.%s.normals.asc" % t)

# Read them.

try:
    rh = open("rh.%s.asc" % surf).readlines()
    rhn = open("rh.%s.normals.asc" % surf).readlines()
    lh = open("lh.%s.asc" % surf).readlines()
    lhn = open("lh.%s.normals.asc" % surf).readlines()
except IOError:
    print >> sys.stderr, "There was a problem generating the .asc files."
    sys.exit(1)

rh_nv = int(rh[1].split()[0])
rhn_nv = int(rhn[1].split()[0])
lh_nv = int(lh[1].split()[0])
lhn_nv = int(lhn[1].split()[0])

if (rh_nv != rhn_nv) or (lh_nv != lhn_nv):
    print >> sys.stderr, "Inconsistent numbers of vertices and normals!"
    sys.exit(1)

# Clean up. The lazy way.

os.system("rm *.asc")

# Now switch to the label dir.

dir = os.path.join(FShome, "subjects", subj, "label")
try:
    os.chdir(dir)
except OSError:
    print >> sys.stderr, "Error locating subject directory %s" % dir
    sys.exit(1)

# Read the cortex label files.

try:
    rhlab = open("rh.cortex.label").readlines()
    lhlab = open("lh.cortex.label").readlines()
    print >> sys.stderr, "Restricting to cortex ..."
    ctxmap_rh = [False] * rh_nv
    ctxmap_lh = [False] * lh_nv
    for s in rhlab[2:]:
        i = int(s[:s.index(' ')])       # convert a space delimited int
        ctxmap_rh[i] = True             # mark this vert as being cortex
    for s in lhlab[2:]:
        i = int(s[:s.index(' ')])
        ctxmap_lh[i] = True

except IOError:
    print >> sys.stderr, "Warning: there was a problem reading the label files."
    ctxmap_rh = [True] * rh_nv
    ctxmap_lh = [True] * lh_nv

# Convert each vertex and normal and write them out.

def doh(nv, h, hn, o, ctxmap):
    for i in range(nv):
        if not ctxmap[i]:
            continue
        # Get the vert and the normal in LPI (ras) order
        v = map(float, h[i+2].split())[0:3]
        n = map(float, hn[i+2].split())[0:3]
        # Translate back to the original origin (o is LPI)
        v[0] += o[0]
        v[1] += o[1]
        v[2] += o[2]
        # convert LPI mm to PRI m
        v = v[1], -v[0], v[2]
        n = n[1], -n[0], n[2]
        for x in v:
            print x / 1000.,
        for x in n:
            print x,
        print

doh(rh_nv, rh, rhn, o, ctxmap_rh)
doh(lh_nv, lh, lhn, o, ctxmap_lh)

