#ifndef LFU_H
#define LFU_H

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <geoms.h>
#include <voxel.h>
#include <DataFiles.h>
#include "samutil.h"

#define ORDER   20          // depends upon the smoothness of the hull (suggest 20th-order)
#define BASIS   ((ORDER + 1) * (ORDER + 1) - 1)
#define MU0_4PI 1.0e-07
#define TINY    1.0e-20
#define INFLATE 0.005       // 5 mm inflation
#define TRUE    1
#define FALSE   0

#define SWAP(a,b)   {double temp=(a);(a)=(b);(b)=temp;}

// 'HULL' -- cortical hull structure (vertices & their normals)
typedef struct {
    FIELD   *vertex;        // vertex[nv] -- coordinate & normal vector of each vertex
    int     nv;             // number of vertices
    double  Vo[3];          // hull origin
    double  scale;          // approximate radius of hull
} HULL;

// integration point structure
typedef struct {
    FIELD   Vi;
    double  Bi[3];
} INT_PNT;

// subroutines
void    ECDFwd(FIELD *Q6, double *Bt, HeaderInfo *Header, ChannelInfo *Channel, HULL *Hull);
void    ECDLeadField(double Org[3], HeaderInfo *Header, ChannelInfo *Channel, HULL *Hull, double **L);
void    ECDLeadFieldElekta(double Org[3], HeaderInfo *Header, ChannelInfo *Channel, HULL *Hull, double **L);
void    ECDGainSolve(double Org[3], double *B, double **C, HeaderInfo *Header, ChannelInfo *Channel, HULL *Hull);
void    ECDSolve(VOXELINFO *Voxel, double *B, double **C, HeaderInfo *Header, ChannelInfo *Channel, HULL *Hull, double *span);
void    ECDSolve2(FIELD *Q6, double *B, double **C, HeaderInfo *Header, ChannelInfo *Channel, HULL *Hull, double *span);
void    ECDSolveFwd(VOXELINFO *Voxel, double *Bc, double *Bu, double **C, HeaderInfo *Header, ChannelInfo *Channel, HULL *Hull, double *span);
void    GetBasis(double Org[3], double scale, double basis[BASIS+1], double gradbasis[BASIS+1][3]);
void    GetCoeffs(INT_PNT *detector, int nchan, HULL *Hull, double **coeffs);
void    GetMeshCoeffs(INT_PNT *detector, int nchan, HULL *Mesh, double **coeffs);
void    GetHull(char *hullName, HULL *Hull);
void    MeshFwd(int ix, HULL *Mesh, HeaderInfo *Header, ChannelInfo *Channel, double *B);
void    NLeadField(VOXELINFO *Voxel, double **L, HeaderInfo *Header, ChannelInfo *Channel, HULL *Hull, int N);
void    NSolveFwd(VOXELINFO *Voxel, double **L, double **B, double **C, int N, int M, int NumVec);
void    NSolveWts(double **B, double **C, double **W, int N, int M);
void    rnynm(double Vs[3], double frnm[ORDER+1][ORDER+1], double finm[ORDER+1][ORDER+1], double fabrnm[ORDER+1][ORDER+1][3], double fabinm[ORDER+1][ORDER+1][3]);
void    xbd(double r[3], double rt[3], double bd[3][3]);
void    xludcmp(double a[BASIS+1][BASIS+1], int indx[BASIS+1], double *d);
void    xlubksb(double a[BASIS+1][BASIS+1], int indx[BASIS+1], double b[BASIS+1]);

#endif
