/* ==========================================================================
 *
 * Author's comments
 *
 * samlib.h
 *
 * ==========================================================================
 */

#ifndef H_SAMLIB
#define H_SAMLIB

#include <stdio.h>
#include <lfulib.h>
#include <mesh.h>
#include <voxel.h>
#include <model.h>
#include <DataFiles.h>
#include <SAMfiles.h>
#include <TimeSegs.h>
#include <nifti1.h>
#include "samutil.h"
#include "sam_param.h"

#define ACTIVE  0
#define CONTROL 1

void    A5toQ6(double *, FIELD *);
int     AtoV(double *, unsigned char *, int, double[2][4][4], FIELD *);
double  Berg(FIELD *Q, double r[3]);
double  BSensor(FIELD *Q, SENSOR *Sensor, double *Sp);
int     BestSphere(char *, ChannelInfo *, int, int);
double  blur(FIELD *, SENSOR *, double *, double);
int     CMELeadField2D(double Org[3], double **, HeaderInfo *Header, ChannelInfo *Channel, int Order);
int     CMELeadField2D5(double Org[3], double **, HeaderInfo *Header, ChannelInfo *Channel, int Order);
int     CMELeadField2D7(double Org[3], double **, HeaderInfo *Header, ChannelInfo *Channel, int Order);
int     CMELeadField3D(double Org[3], double **, HeaderInfo *Header, ChannelInfo *Channel, int Order);
int     CMEMom2D(double Org[3], double Mom[7], double **C, HeaderInfo *Header, ChannelInfo *Channel, int Order, double *span);
int     CMESolve2D(double Org[3], double *Bt, double **C, HeaderInfo *Header, ChannelInfo *Channel, int Order, double *span);
int     CMESolve3D(double Org[3], double *Bt, double **C, HeaderInfo *Header, ChannelInfo *Channel, int Order, double *span);
int     CMESolveB2D(double Org[3], double **B, double **C, HeaderInfo *Header, ChannelInfo *Channel, int Order, double *span);
int     CMESolveB2D_3(double Org[3], double **B, double **C, HeaderInfo *Header, ChannelInfo *Channel, int Order, double *span);
int     CMESolveB2D5(double Org[3], double **B, double **C, HeaderInfo *Header, ChannelInfo *Channel, int Order, double *span);
int     CMESolveB3D(double Org[3], double **B, double **C, HeaderInfo *Header, ChannelInfo *Channel, int Order, double *span);
int     CopySensor(SENSOR *, SENSOR *);
int     CtoI(int *, int, int, ChannelInfo *, char **);
void    CtoS(double *Vc, double *Vs);
void    CtoSl(double *, double *);
double  Dflip(void *);
int     DIPLeadField2D(double Org[3], double **, HeaderInfo *Header, ChannelInfo *Channel, int Order);
int     DIPLeadField3D(double Org[3], double **, HeaderInfo *Header, ChannelInfo *Channel, int Order);
int     DIPMom3D(double Org[3], double Mom[3], double **C, HeaderInfo *Header, ChannelInfo *Channel, int Order, double *span);
void    DIPSolve2D(VOXELINFO *Voxel, double *Bt, double **C, HeaderInfo *Header, ChannelInfo *Channel, int Order, double *span);
void    DIPSolve3D(VOXELINFO *Voxel, double *Bt, double **C, HeaderInfo *Header, ChannelInfo *Channel, int Order, double *span);
int     DipoleLeadField2D(double Org[3], double **L, double Vx[3], double Vy[3], HeaderInfo *Header, ChannelInfo *Channel, int Order);
int     DipoleMoment2D(double Org[3], double **L, double Mom[3], double **C, HeaderInfo *Header, ChannelInfo *Channel, int Order, double *span);
int     DQSolve3D(double Org[3], double *Bt, double **C, HeaderInfo *Header, ChannelInfo *Channel, int Order, double *span);
void    DSwap(double *, double *);
void    DualLeadField3D(double Da[3], double Db[3], double **L, HeaderInfo *Header, ChannelInfo *Channel, int Order);
void    DualECDSolve3D(double Da[3], double Db[3], double **B, double **C, HeaderInfo *Header, ChannelInfo *Channel, int Order, double *span);
void    DualScalarSolve(double **C, double **Bp, double **Wgt, int rank);
void    DualWeightSolve(double **C, double **B, double **W, int rank);
void    EEGLeadField(double r[3], double **e, int M, double **L);
void    EEGSolve(VOXELINFO *Voxel, double *Vt, double **C, double **e, int M, double *span);
double  eps(double);
void    ESum(double *, double *, double *);
void    EtoR(double[3], double[3][3]);
float   Fflip(void *);
int     FitJig(FIELD *, FIELD *, int, XFORM *, double *);
int     FixQ(SNODE *, int, FIELD *, int, int);
float   FloatSwap(float);
double  Forward(FIELD *, SENSOR *, double *);
double  Fwd(FIELD *, SENSOR *, double *, int);
double  Fwd2(FIELD *, SENSOR *, int);
double  FwdLine(FIELD *, double, SENSOR *, int);
int     FwdLineVec(FIELD *, double, double *, ChannelInfo *, int *, int, int, int);
double  FwdM(FIELD *, SENSOR *, int);
int     FwdVec(FIELD *, double *, HeaderInfo *, ChannelInfo *, int);
int     FwdMVec(FIELD *, double *, HeaderInfo *, ChannelInfo *, int);
void    FreeSensor(SENSOR *);
int     GJ(double **, int, double *);
int     GetBad(char *, int **, int *);
int     GetBadSegments(char *, int, int, double, unsigned char ***, int *, int);
void    GetCov(char *, COV_HDR *, int **, double ***);
int     GetCov2(char *, COV_HDR *, int *, double **);
void    GetDsInfo(char *, HeaderInfo *, ChannelInfo **, EpochInfo **, unsigned char ***, int);
void    GetDsData(HeaderInfo *, int, int, double, double *);
int     GetCTFData(HeaderInfo *, int, int, double, double *);
int     GetHC(char *, double (*)[3]);
void    GetHDM(char *, HEAD_MODEL *);
int     GetMagnesChan(char *, char *, HeaderInfo *, ChannelInfo *, int, int, double *);
int     GetMagnesData(char *, char *, HeaderInfo *, ChannelInfo *, int, double **);
int     GetMagnesInfo(char *, char *, HeaderInfo *, ChannelInfo **, EpochInfo **);
int     GetMagnesMarkers(char *, char *, HeaderInfo *, ChannelInfo *, SAM_MARKS **, int *);
int     GetMagnesWts(char *, char *, HeaderInfo *, ChannelInfo *);
void    GetMarkers(char *, SAM_MARKS **, int *);
int     GetMEG(char *, HeaderInfo *, ChannelInfo **, EpochInfo **, long ****, int);
void    GetMesh(char *MeshPath, HULL *Mesh);
void    GetNIFTI(char *ImageName, int *nt, float ***Data, nifti_1_header *Header);
void    GetNIFTIWts(char *WgtFilePath, double ***Wgt, int *NumChans, int *NumVoxels, nifti_1_header *Header, char *Ext, unsigned char **ExtHeader);
void    GetNIFTIMask(char *MaskPath, unsigned char **Mask, int *nv);
void    GetNoise(char *, char *, double *);
void    GetParams(char *, PARMINFO * Params);
void    GetMRIPath(char *path, int len, PARMINFO *p, char *name, int exist);
int     GetSpecFile(char *, COV_SPEC **, int *, int *);
void    GetTargets(char *PathName, VOXELINFO **Target, int *N);
int     GetTmp(char *, long int *, int, int);
int     GetWH2500Data(char *, char *, HeaderInfo *, int, int, double, double *);
int     GetWH2500Info(char *, char *, HeaderInfo *, ChannelInfo **, EpochInfo **);
void    GetVoxels(VOXELINFO **Voxel, int *NumVoxels, HULL * Hull, double Ts[3], double Te[3], double step);
void    GetVoxelsFS(char *Atlas, VOXELINFO **Voxel, int *NumVoxels, double Ts[3], double Te[3], double *step);
void    GetVoxelsHDM(VOXELINFO **Voxel, int *NumVoxels, char *ShapeName, double Ts[3], double Te[3], double step);
void    GetVoxelsHS(VOXELINFO **Voxel, int *NumVoxels, char *RunName, double Ts[3], double Te[3], double step);
void    GetWts(char *, SAM_HDR *, int **, double ***);
void    HStoMS(char *, HeaderInfo *, ChannelInfo *);
int     HullToMS(char *, HeaderInfo *, ChannelInfo *);
double  **InitElect(char *, int *);
PROBE   *InitProbe(char *);
void    ItoC(int *, int, ChannelInfo *, char **);
void    J1toB(double O[3], double S[3], double Sp[3], double L[3][3]);
void    J2toB(double O[3], double S[3], double Sp[3], double L[3][3][3]);
void    JMtoB(double Org[3], double S[3], double Sp[3], double L[12][3]);
int     KENsolve(FIELD *, double *, double **, HeaderInfo *, ChannelInfo *, int);
double  KENsolve2(FIELD *, double *, double **, HeaderInfo *, ChannelInfo *, int);
int     KENsolveX(FIELD *, double *, double *, double **, HeaderInfo *, ChannelInfo *, int);
int     KENsolve2D(FIELD *, double *, double **, HeaderInfo *, ChannelInfo *, int, double *);
int     KENsolve2DI(FIELD *, double, double *, double **, ChannelInfo *, int *, int, int, int);
int     KENsolve3D(FIELD *, double *, double **, HeaderInfo *, ChannelInfo *, int, double *);
int     KENsolve3DI(FIELD *, double, double *, double **, ChannelInfo *, int *, int, int, int);
int     LeadField(SENSOR *, double *, double *, double *, int, double *);
long    LongSwap(long);
double  LSensor(FIELD *, double, SENSOR *);
int     LSolve(BNODE *, int, FIELD *, double, double *, double *);
void    LtoB(FIELD *, double, FIELD *);
void    LUBackSub(double **, int, int *, double *);
int     LUDecomp(double **, int, int *);
int     MAmoeba(FIELD *, ChannelInfo *, SVD *, double);
SENSOR  *MakeSensor(PROTO *, FIELD *);
int     ml1(SHAPEINFO *, int, double *, double *, double *);
void    ml2(SHAPEINFO *, int, double *, double **, double *, double *);
int     MLCoeff(SNODE *, int, double *, double **, double *, double *);
int     MLPass(SNODE *, int, double *, double *, double *);
double  MSensor(FIELD *, SENSOR *);
int     MaxSize(FIELD *, double **, SVD *);
int     MeanVector(SNODE *, int, int, double *, double *);
void    MeshCorr(HeaderInfo *Header, ChannelInfo *NewChannel, double **C, double Noise, HULL *Hull, MESHINFO *Mesh, int N, int *nv, double *Correlation);
void    MeshScatter(HeaderInfo *Header, ChannelInfo *NewChannel, double **C, double Noise, HULL *Hull, MESHINFO *Mesh, int N, int *nv, double *Correlation, double *rms, int evaluate);
void    MeshScatter2D(HeaderInfo *Header, ChannelInfo *NewChannel, double **C, double Noise, MESHINFO *Mesh, int N, int *nv, double *Correlation);
void    MoveField(FIELD *, XFORM *);
void    MoveFrame(HeaderInfo *Header, ChannelInfo *InChannel, ChannelInfo *OutChannel, char *TransFile);
void    MoveMEG(ChannelInfo *, ChannelInfo *, int *, int, XFORM *);
int     MoveProbe(PROBE *, XFORM *);
void    MoveSensor(SENSOR *, XFORM *);
int     MSolve(BNODE *, int, FIELD *, double *);
int     MSolveLoop(BNODE *, int, FIELD *, double, double *, double *);
int     MSolveSensor(SENSOR *, double, FIELD *, double, double *, double *, int, double *);
void    MtoB(FIELD *, FIELD *);
int     MultiLeadField3D(double Org[3], double Vx[3], double **L, HeaderInfo * Header, ChannelInfo * Channel, int Order);
int     MultiSolve3D(double Org[3], double **B, double **C, HeaderInfo * Header, ChannelInfo * Channel, int Order, double *span, int iterate);
double  MutFwd(FIELD *, double, SENSOR *, int);
void    NLeadField2D(VOXELINFO *Voxel, double **L, HeaderInfo *Header, ChannelInfo *Channel, int N);
void    NLeadField3D(VOXELINFO *Voxel, double **L, HeaderInfo *Header, ChannelInfo *Channel, int N);
void    octo(FIELD Org, double dr, double dt, double **Seven);
FILE    *OpenDsFile(HeaderInfo *, char *);
int     OrthoLeadField2D(double Org[3], double **L, double **Pd, double **Pq, HeaderInfo * Header, ChannelInfo * Channel, int Order);
int     OrthoLeadField3D(double Org[3], double **L, double **Pd, double **Pq, HeaderInfo * Header, ChannelInfo * Channel, int Order);
int     OrthoPlane(double *Vr, double *Vx, double *Vy);
int     OrthoSolve2D(double Org[3], double **B, double **C, double **Cp, HeaderInfo * Header, ChannelInfo * Channel, int Order, double *span);
int     OrthoSolve3D(double Org[3], double **B, double **C, HeaderInfo * Header, ChannelInfo * Channel, int Order, double *span);
void    P5toP6(FIELD *, FIELD *);
void    P6toP5(FIELD *, FIELD *);
void    ParseFileName(char *, char *);
int     ParsePath(char *, char *);
void    pinv(double **, int, int);
int     PosProbe(PROBE *, char *, int);
double  PredictPoint(SHAPEINFO *, double *);
int     PutCTFData(char *, HeaderInfo *, double, double *);
int     PutMEG(char *, HeaderInfo *, ChannelInfo *, EpochInfo *, long ***);
void    PutCov(char *, COV_HDR, int *, double **);
void    PutTmp(char *, long int *, int);
void    PutWts(char *WgtPath, double *Wgts, HeaderInfo Header, COV_HDR CovHdr, int V);
void    PutNIFTITargets(char *WgtFilePath, double **Wgt, float Noise, int M, int V);
void    PutNIFTIWts(char *WgtFilePath, PARMINFO *Params, double **Wgt, float Noise, int M);
int     Q0Solve(SNODE *, int, FIELD *, double *);
int     Q1Solve(SNODE *, int, FIELD *, double *);
void    Q0toB(FIELD *, FIELD *);
double  Q0toV(FIELD *, double *, double);
void    Q6toA5(FIELD *, double *);
int     QAmoeba(FIELD *, SNODE *, int);
int     QDPLeadField2D(double Org[3], double **L, HeaderInfo * Header, ChannelInfo * Channel, int Order);
int     QDPLeadField3D(double Org[3], double **L, HeaderInfo * Header, ChannelInfo * Channel, int Order);
int     QDPSolve2D(double Org[3], double *Bt, double **C, HeaderInfo * Header, ChannelInfo * Channel, int Order, double *span);
int     QDPSolve3D(double Org[3], double *Bt, double **C, HeaderInfo * Header, ChannelInfo * Channel, int Order, double *span);
int     QGuess(SNODE *, int, FIELD *);
int     QSolve(SNODE *, int, FIELD *, int, int, double *);
int     QStoB(FIELD *, TGLS *, double, FIELD *);
double  QStoV(FIELD *, double *);
void    QtoB(FIELD *, FIELD *, double *);
double  *QtoV(FIELD *, TGLS *, double);
double  QVar(SNODE *, int);
double  QxCorr(SNODE *, int);
double  RMSError(SNODE *, int);
double  RMSSignal(SNODE *, int);
void    RSum(double[3][3], double[3][3], double[3][3]);
double  RandomGauss(double, double);
void    RtoE(double[3][3], double[3]);
void    SAM2NIFTIHdr(PARMINFO *Params, int nt, float time_start, float time_step, char *intent_name, int xfm, nifti_1_header *Header);
int     SAMFwd(double **, double *, double, double *, int);
int     SAMsolve(SVD *, double **, double *, double *, double *, double *, double);
void    SAMsolve2(double **, double *, double *, double, int, double *, double *);
void    SERvs(double **x, double *vs, double *Bp, int TS, int TE, int M, double tau, double rate, double noise);
void    Shuffle(int *, int);
int     SLSolve(SENSOR *, double *, FIELD *, double, double, double *, double *, int, double *);
int     SSolve(SENSOR *, double *, FIELD *, double *, double *, int, double *);
void    SVDAllocate(SVD *);
int     SVDBackSub(SVD *, double *, double *, double);
void    SVDFree(SVD *);
int     SVDinvert(SVD *, double **, double);
int     SVDSubSpace(SVD *, double *, double *, double);
short   ShortSwap(short);
void    ShowProbe(PROBE *);
int     SolveECM(double **C, double **B, double **W, double noise2, int M, double P[2], double N[2]);
int     SolveECM_3(double **C, double **B, double **W, double noise2, int M, double P[3], double N[3]);
int     SolveJoint(double **C, double **B, double *Bp, int rank);
int     SolveWeights(double **, double *, int, double, int, double *, double *);
void    Sort(double *, int);
int     StoB(FIELD *, SNODE *, int);
void    StoC(double *, double *);
void    StoCl(double *, double *);
void    svd(SVD *);
int     svdl(SVD *);
int     VAmoeba(FIELD *, double **, SVD *, double);
int     VSolve(FIELD *, double **, SVD *);

#endif // H_SAMLIB
