#ifndef H_SIGLIB
#define H_SIGLIB

#include <filters.h>
#include <complex.h>
#include <geoms.h>
#include <fftw3.h>
#include "samutil.h"

#define BASE_MEAN   0
#define BASE_TREND  1
#define BASE_QUAD   2
#define BASE_CUBE   3

void    balanc(double **, int);
void    bdiir(double *, double *, int, IIRSPEC *);
double  betacf(double, double, double);
double  betai(double, double, double);
void    Butterworth(FFTSPEC *Filter, int N, double HighPassFreq, double LowPassFreq, double SampleRate, int FilterType, int Order, double *bw);
int     chirp(double *, int, double, double, double, double);
int     choldc3(double (*)[3], double *);
void    cholinv3(double A[3][3]);
int     cholinv4(double A[4][4]);
int     cholinv5(double A[5][5]);
int     cholinv7(double A[7][7]);
int     cholinv8(double A[8][8]);
int     cholinv9(double A[9][9]);
int     cholinv11(double A[11][11]);
int     cholinv12(double A[12][12]);
int     cholinv(double **, int);
void    comb(double *, int, double, double);
int     corr(double **, int, int, int, int, double **);
int     cov(double **, int, int, int, int, double **);
int     cspline(double *, double *, int, int);
int     debase(double *, int, int);
void    deigen(double **, double *, int);       // eigendecomposition with sorting
void    deigen3(double A[3][3], double B[3]);
void    deigen4(double A[4][4], double B[4]);
void    deigen5(double A[5][5], double B[5]);
void    deigen6(double A[6][6], double B[6]);
void    deigen7(double A[7][7], double B[7]);
void    deigen8(double A[8][8], double B[8]);
void    deigen9(double A[9][9], double B[9]);
void    deigen11(double A[11][11], double B[11]);
void    demean(double *, int);
int     dequad(double *, int);
int     denoise(double **, int, int *, int, int *, int);
void    derivs(double x, double *y, double dydx);
void    detect(double *x, int N, IIRSPEC *Filter);
void    detrend(double *, int);
int     dft(double *, double *, int, double, double, double, double);
void    dswap(double *, double *);
void    eigen(double **, double *, int);
void    eigenl(double **, double *, int);
int     eigen3(double A[3][3], double B[3]);            // no sort
int     eigen4(double A[4][4], double B[4]);            // no sort
int     eigen5(double A[5][5], double B[5]);            // no sort
void    eigen6(double A[6][6], double B[6]);            // no sort
int     eigen7(double A[7][7], double B[7]);            // no sort
int     eigen8(double A[8][8], double B[8]);            // no sort
int     eigen11(double A[11][11], double B[11]);        // no sort
int     eigen12(double A[12][12], double B[12]);        // no sort
void    eig22(double e[2][2], double l[2]);
int     eig33(double (*)[3], double *);
void    equalizer(double *data,double *result, FFTSPEC *filter, double fhi, double flo, double srate, int len);
double  erf(double);
double  erfc(double);
double  evlmem(double, double *, int, double);
void    evolve(double *x, double *h, int TS, int TE, double LPFreq, double tau, double rate);
int     extendts(double *, double *, int, int);
void    fft(int len, double *data, double *result);
void    FFTfilter(double *data, double *result, FFTSPEC *Filter, int len);
int     filter(double *, double *, int, IIRSPEC *);
void    fir(double *, double *, int);
int     fixrts(double *, int);
int     fmf(double *, double *, int, int);
void    ForwardDft(double _Complex *, double _Complex *, int);
double  freqinfo(double *x, double *y, int TS, int TE, double HPFreq, double LPFreq, double rate);
double  gammln(double);
double  gammp(double, double);
double  gammq(double, double);
int     gcf(double *, double, double, double *);
void    geig22(double (*)[2], double (*)[2], double (*)[2], double *);
int     gser(double *, double, double, double *);
void    Hanning(double *, int);
void    hilbert(int len, double *data, double *result);
void    iir(double *, double *, int, IIRSPEC *);
int     inv22(double A[2][2]);
int     inv33(double A[3][3]);
void    invert(double **, int);
void    invertl(double **, int);
double  kurtosis(double *, int, int);
int     laguerre(double _Complex *, int, double _Complex *, double, int);
void    mab(double *, int, double *, double *);
void    sam(double **, double *, int, int, double *);
double  legendre(int, double);
int     lpgap(double *, double *, int, int, int, double *, int, int);
int     marker(double **, int, double, int *, int, double, double, double *);
double  median(double *, int);
void    memcof(double *, int, int, double *, double *);
void    mkfilter(IIRSPEC *, int, double, double, double, double *);
void    mkiir(IIRSPEC *);
void    mknotch(FFTSPEC *, double HPFreq, double LPFreq, double Rate, int N, double Mains);
double  mode(double *, int, int);
void    Morlet(double *, int);
int     mr1df(double, double*, double*, int);
void    mutinfo(double *x, double *y, double *Info, int TS, int TE, double LPx, double LPy, double tau, double rate);
double  mutinfostat(double *x, double *y, double LPx, double LPy, int TS, int TE, double rate);
int     notch(double *, int, double, double);
int     outlier(double *, double *, int, double *, int);
double  pax(double *, double *, int, int);
double  peak(double *, int, double, double, double, double);
double  PeakSlope(double *, int);
void    phevolve(double *x, double *h, int TS, int TE, double LPFreq, double tau, double rate);
double  power(double *, int, int);
double  PpRms(double *, int);
int     predict(double *, double *, int, int, double *, int, int);
void    predictability(double *x, double *lambda, int TS, int TE, int W, int F, int P);
int     ProbF(double, int, double, int, double *, double *);
int     ProbT(double, double, int, double, double, int, double *, double *);
int     ProbZ(double, double, int, double *, double *);
void    pwr(double *x, double *p, int TS, int TE, double HPFreq, double rate);
void    QtoM(QUATERNION quaternion, double Rotate[3][3]);
void    rk4(double *y, double *dydx, int n, double x, double h, double *yout, void (*derivs)(double, double *, double *));
void    rkdumb(double *vstart, int nvar, double x1, double x2, int nstep, void (*derivs)(double, double *, double *));
int     realfft(double, double *, double *, int);
double  response(IIRSPEC *, double, double);
double  rms(double *, int);
int     robavg(double ***, double **, double **, double *, double, int, int, int, int *, int *);
int     robvar(double ***, double **, double **, double *, double, int, int, int);
void    set_wisfile(void);
double  slope(short int *, int, int, int, int);
int     SolveEigen(double **A, double **B, double *Vec, double *Val, int N, int MinMax);
int     SolveEigen5(double A[5][5], double B[5][5], double Y[5], double V[5]);
void    SolveEigen6(double A[6][6], double B[6][6], double Y[6], double V[6]);
int     SolveEigen7(double A[7][7], double B[7][7], double Y[7], double V[7]);
int     SolveEigen11(double A[11][11], double B[11][11], double Y[11], double V[11]);
void    st(int n, int lo, int hi, double *data, double *result);
int     st_freq(double f, int len, double srate);
void    tred2(double **, int, double *, double *);
int     tqli(double *, double *, int, double **);
double  vfcorr(float *, float *, int);
double  vfdist(float *, float *, int);
int     vlag(double **, double **, double *, int, int, int, int, int, double, double *);
double  vpax(double **, double **, int, int, int);
double  vxcorr(double **, double **, int, int, int);
double  vxreg(double **, double **, int, int, int);
void    whiten(double **x, double **y, int N, int *K, int T);
int     widrow(double *, int, double, double, double);
int     widrowBi(double *, int, double, double, double);
double  xCorr(double **, double **, int, int);
int     xmtx(double **, int, int, int *, int, int *, int, double **, double **, double *, double **);
void    YtoR(double theta, double rotation[3][3]);
int     zpf(double *, double *, int, IIRSPEC *);
int     zroots(double _Complex *, int, double _Complex *, int);

#if !defined(log2)
double  log2(double x); /* sometimes this is not in math.h */
#endif

#endif  // H_SIGLIB

